# osh
# Copyright (C) 2005 Jack Orenstein <jao@geophile.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

"""Usage: select FUNCTION

Receives a stream of objects through stdin and passes on to stdout
those objects for which FUNCTION evalutes to true. For example,
if stdin contains tuples containing a person's name and age, e.g.
('hannah', 11), ('julia', 6), then here's how we can pass on only
those tuples for which age >= 18:

... ^ select 't: t[1] >= 18' ^ ...
"""

import osh.oshfunction
import osh.oshobjects

def select(*args):
    return _Select().process_args(*args)

class _Select(osh.oshobjects.Op):

    _oshfunction = None


    # object interface

    def __init__(self):
        osh.oshobjects.Op.__init__(self, '', (1, 1))


    # OshCommand interface
    
    def doc(self):
        return __doc__

    def setup(self):
        args = self.args()
        self._oshfunction = args.next_function()
        if self._oshfunction is None or args.has_next():
            self.usage()


    # Receiver interface
    
    def receive(self, object):
        if self._oshfunction(object):
            self.send(object)
