# osh
# Copyright (C) 2005 Jack Orenstein <jao@geophile.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import sys
import types

from oshargs import Option
from osherror import OshError
import oshapiparser
import oshobjects
import command
import command.f

class _ReturnList:
    pass

def debug(verbosity):
    oshobjects.verbosity = verbosity

for module_name in command.__all__:
    exec('import command.%s' % module_name)
    mod = getattr(command, module_name)
    for element_name in dir(mod):
        if not element_name.startswith('_'):
            element = getattr(mod, element_name)
            if type(element) is types.FunctionType:
                globals()[element_name] = element


# Top-level oshapi invocation

def osh(*ops):
    # Do we need to accumulate output?
    output_accumulator = None
    if (ops[-1].__class__ is _ReturnList):
        output_accumulator = []
        ops = ops[:-1]
    # Construct the pipeline
    pipeline = oshapiparser._sequence_op(ops)
    # Append accumulator if we're accumulating output
    if output_accumulator is not None:
        pipeline.append_opset(oshobjects.Opset(f(lambda *x: output_accumulator.append(x))))
    # Run
    pipeline.setup()
    if oshobjects.verbosity >= 1:
        print pipeline.dump()
    try:
        pipeline.execute()
    except oshobjects.ErrorHandlerException, e:
        raise e.reason
    return output_accumulator

# consecutive is used by both sort and unique. For sort, it supplies a function argument.
# For unique, it's just a flag.

def consecutive(function_spec = True):
    return Option('-c', function_spec)

class IllegalOshCommand(Exception):

    def __init__(self, garbage):
        self._garbage = garbage

    def __str__(self):
        return 'Illegal osh command or sub-command: %s' % self._garbage

# If present as last argument to osh(), indicates that we should accumulate 'o' stream output.
def return_list():
    return _ReturnList()
