# osh
# Copyright (C) 2005 Jack Orenstein <jao@geophile.com>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

import inspect
import types

import oshobjects

def _is_base_op(x):
    for klass in inspect.getmro(x.__class__):
        if klass is oshobjects.BaseOp:
            return True
    return False

def _is_map(x):
    return type(x) is types.DictType

def _is_sequence(x):
    return type(x) is types.ListType or type(x) is types.TupleType

def _op(x):
    if _is_base_op(x):
        op = x
    elif _is_sequence(x):
        op = _sequence_op(x)
    else:
        raise IllegalOshCommand(x)
    return op

def _opset(x):
    if _is_base_op(x):
        opset = oshobjects.Opset(x)
    elif _is_sequence(x):
        opset = oshobjects.Opset(_sequence_op(x))
    elif _is_map(x):
        opset = _map_opset(x)
    else:
        raise IllegalOshCommand(x)
    return opset

def _map_opset(map):
    opset = oshobjects.Opset()
    for label, x in map.iteritems():
        op = _op(x)
        op.set_label(label)
        opset.add_op(op)
    return opset

def _sequence_op(sequence):
    pipeline = oshobjects.Pipeline()
    for x in sequence:
        pipeline.append_opset(_opset(x))
    return pipeline

class IllegalOshCommand(Exception):

    def __init__(self, garbage):
        self._garbage = garbage

    def __str__(self):
        return 'Illegal osh command or sub-command: %s' % self._garbage
